@@ -94,7 +94,7 @@ w <- function(l) l / sum(l)
 
 gls2 <- function(..., cor_fun, tree) {
   for (par in seq(1, 0, -0.05)) {
-    m <- purrr::safely(~nlme::gls(..., correlation = cor_fun(par, .x)))(tree)
+    m <- purrr::safely(function(.x, ...) nlme::gls(..., correlation = cor_fun(par, .x)))(tree, ...)
     if (is.null(m$error)) break
   }
   return(m)
